window.Gleam.initSearch([{"doc":"validate_monadic","title":"validate_monadic","content":"# Monadic Validation in Gleam!\n\n```sh\ngleam add validate_monadic\n```\n\n```gleam\nimport validate_monadic as validate\n```\n\n# Usage\n\nThis module is the minimal set of functions around a single type alias for `Result` so it may \nbe used as a &quot;validation monad&quot;.\n\nThe `/examples` folder contains two examples for full usage of this module, using\nevery method contained in it.\n\n[Checkout out the examples here](https://github.com/abradley2/gleam-validate/tree/master/examples/src)\n\n## Development\n\n```sh\ngleam run   # Run the project\ngleam test  # Run the tests\ngleam shell # Run an Erlang shell\n```\n","url":"index.html"},{"doc":"validate_monadic","title":"ErrorList","content":"pub type ErrorList(error) =\n  #(error, List(error))\n Simple type alias for a non-empty list. A non-empty list is just a structure containing the\n first item of the list, followed by the rest of the list. You can find a more \n useful implementation [here](https://hexdocs.pm/non_empty_list/non_empty_list.html#NonEmptyList)\n . For the purposes of not including an extra dependency, we just use a tuple here.\n\nSynonyms:\nvalidate_monadic.ErrorList\nvalidate_monadic ErrorList","url":"validate_monadic.html#ErrorList"},{"doc":"validate_monadic","title":"Validation","content":"pub type Validation(validated, error) =\n  Result(validated, ErrorList(error))\n Simple type alias over a Result with a *non-empty* list off generic errors\n for the Error branch. The non-empty list is important here. For validation we must\n represent a Result type that can have multiple errors, but we must avoid allowing \n something like `Error([])` in which we can have an error branch but no errors!\n\nSynonyms:\nvalidate_monadic.Validation\nvalidate_monadic Validation","url":"validate_monadic.html#Validation"},{"doc":"validate_monadic","title":"and_also","content":"pub fn and_also(\n  validation_a: Result(a, #(b, List(b))),\n  validation_b: Result(a, #(b, List(b))),\n) -&gt; Result(a, #(b, List(b)))\n Combine two validation results into one. This mainly for merging errors. The `Ok `branch of the\n last validation supplied will be the returned `Ok` branch. This is used internally by `compose`\nSynonyms:\nvalidate_monadic.and_also\nvalidate_monadic and_also","url":"validate_monadic.html#and_also"},{"doc":"validate_monadic","title":"and_map","content":"pub fn and_map(\n  prev prev_validation: Result(fn(a) -&gt; b, #(c, List(c))),\n  next validation: Result(a, #(c, List(c))),\n) -&gt; Result(b, #(c, List(c)))\n Used to create applicative chains of validation. This is very important for combining validation\n of fields into the validation of an entire form.\n \n \n let validate_form = function.curry3(fn (\n     ValidFirstName,\n     ValidLastName,\n     ValidAge\n   ) {\n     ValidForm(ValidFirstName, ValidLastName, ValidAge)\n   })\n \n let validation_result =\n   validate.succeed(validate_form)\n   |&gt; validate.and_map(first_name_result)\n   |&gt; validate.and_map(last_name_result)\n   |&gt; validate.and_map(age_result)\n \nSynonyms:\nvalidate_monadic.and_map\nvalidate_monadic and_map","url":"validate_monadic.html#and_map"},{"doc":"validate_monadic","title":"and_then","content":"pub fn and_then(\n  over validation: Result(a, #(b, List(b))),\n  bind bind_fn: fn(a) -&gt; Result(c, #(b, List(b))),\n) -&gt; Result(c, #(b, List(b)))\n Specify a validation that will run after a given validation, using its result. This is very\n useful for validations that need to run after a transform is attempted.\n \n \n let age_result =\n   form.age_string\n   |&gt; is_parsable_int\n   |&gt; validate.and_then(int_less_than(_, 101))\n \n \n It can easily be used in conjuction with `compose`\n \n \n let age_result =\n   form.age_string\n   |&gt; is_parsable_int\n   |&gt; validate.and_then(\n     validate.compose(int_less_than(_, 101), [int_greater_than(_, 0)])\n   )\n \nSynonyms:\nvalidate_monadic.and_then\nvalidate_monadic and_then","url":"validate_monadic.html#and_then"},{"doc":"validate_monadic","title":"compose","content":"pub fn compose(\n  input: a,\n  validation: fn(a) -&gt; Result(b, #(c, List(c))),\n  validations: List(fn(a) -&gt; Result(b, #(c, List(c)))),\n) -&gt; Result(b, #(c, List(c)))\n Compose together multiple validations. This combine the errors of all validations that fail,\n and does not stop at the first failure. Takes the input to be validated as the first argument,\n then a non-empty list of unary functions that take that input and return a validation result\n of the same type.\n \n \n \n \n let validation_result =\n   raw_field\n     |&gt; validate.compose(no_numbers, [shorter_than_10, no_symbols, no_whitespace])\n \nSynonyms:\nvalidate_monadic.compose\nvalidate_monadic compose","url":"validate_monadic.html#compose"},{"doc":"validate_monadic","title":"error","content":"pub fn error(err: a) -&gt; Result(b, #(a, List(a)))\n Convenience function for lifting a single error into our non-empty list error.\nSynonyms:\nvalidate_monadic.error\nvalidate_monadic error","url":"validate_monadic.html#error"},{"doc":"validate_monadic","title":"map","content":"pub fn map(\n  over validation: Result(a, #(b, List(b))),\n  with map_fn: fn(a) -&gt; c,\n) -&gt; Result(c, #(b, List(b)))\n Map a validation type to another type. This is often useful to nest the result of a validation \n into a &quot;ValidatedType&quot;. This is just an alias over `result.map`\n \n \n   pub type ValidatedLastName {\n     ValidatedLastName(String)\n   }\n \n   // ...\n \n   let last_name_result =\n     form.last_name\n       |&gt; string_non_empty\n       |&gt; validate.map(ValidatedLastName)\n \nSynonyms:\nvalidate_monadic.map\nvalidate_monadic map","url":"validate_monadic.html#map"},{"doc":"validate_monadic","title":"map_error","content":"pub fn map_error(\n  over validation: Result(a, #(b, List(b))),\n  with map_fn: fn(b) -&gt; c,\n) -&gt; Result(a, #(c, List(c)))\n Map over all the errors for a validation result.\n This is very useful for cases where you have re-usable validators with generic error messages,\n and you wish to specify the errors are associated with a specific field\n \n \n \n let validation_result =\n   raw_field\n     |&gt; validate.compose(no_numbers, [shorter_than_10])\n     |&gt; validate.map_error(string.append(&quot;Field Name Error: &quot;, _))\n \nSynonyms:\nvalidate_monadic.map_error\nvalidate_monadic map_error","url":"validate_monadic.html#map_error"},{"doc":"validate_monadic","title":"succeed","content":"pub fn succeed(a: a) -&gt; Result(a, #(b, List(b)))\n Convenience function for lifting a value into our validation type&#39;s `Ok` branch, like other methods\n in this module, it is literally just an alias for a `Result` type method. \nSynonyms:\nvalidate_monadic.succeed\nvalidate_monadic succeed","url":"validate_monadic.html#succeed"},{"doc":"validate_monadic","title":"validate_monadic","content":"","url":"validate_monadic.html"}]);